@@ -9,7 +9,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '3.0008_1';
+$VERSION = '3.0001';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -29,7 +29,7 @@ sub driver{
 				   'Version' => $VERSION,
 				   'Err'    => \$DBD::mysql::err,
 				   'Errstr' => \$DBD::mysql::errstr,
-				   'Attribution' => 'DBD::mysql by Patrick Galbraith'
+				   'Attribution' => 'DBD::mysql by Rudy Lippan'
 				 });
 
     $drh;
@@ -473,7 +473,7 @@ DBD::mysql - MySQL driver for the Perl5 Database Interface (DBI)
     @databases = DBI->data_sources("mysql");
        or
     @databases = DBI->data_sources("mysql",
-      {"host" => $host, "port" => $port, "user" => $user, password => $pass});
+				   {"host" => $host, "port" => $port});
 
     $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
        or
@@ -590,7 +590,7 @@ statement handle with:
 This statement handle can be used for multiple things. First of all
 you can retreive a row of data:
 
-  my $row = $sth->fetchrow_hashref();
+  my $row = $sth->fetchow_hashref();
 
 If your table has columns ID and NAME, then $row will be hash ref with
 keys ID and NAME. See L<STATEMENT HANDLES> below for more details on
@@ -621,8 +621,8 @@ A C<database> must always be specified.
 
 =item port
 
-The hostname, if not specified or specified as '' or 'localhost', will
-default to an MySQL daemon running on the local machine using the default
+The hostname, if not specified or specified as '', will default to an
+MySQL daemon running on the local machine on the default port
 for the UNIX socket.
 
 Should the MySQL daemon be running on a non-standard port number,
@@ -630,8 +630,6 @@ you may explicitly state the port number to connect to in the C<hostname>
 argument, by concatenating the I<hostname> and I<port number> together
 separated by a colon ( C<:> ) character or by using the  C<port> argument.
 
-To connect to a MySQL server on localhost using TCP/IP, you must specify the
-hostname as 127.0.0.1 (with the optional port).
 
 =item mysql_client_found_rows
 
@@ -742,25 +740,22 @@ disallow LOCAL.)
 
 =item Prepared statement support (server side prepare)
 
-As of 3.0002_1, server side prepare statements are on by default (if your
-server is >= 4.1.3)
-
-To use driver emulated prepared statements, all you need to do is set the variable 
-mysql_emulated_prepare in the connect:
+To use server side prepared statements, all you need to do is set the variable 
+mysql_server_prepare in the connect:
 
 $dbh = DBI->connect(
-                    "DBI:mysql:database=test;host=localhost;mysql_emulated_prepare=1",
+                    "DBI:mysql:database=test;host=localhost:mysql_server_prepare=1",
                     "",
                     "",
                     { RaiseError => 1, AutoCommit => 1 }
                     );
 
-* Note: delimiter for this param is ';'
-
 To make sure that the 'make test' step tests whether server prepare works, you just
 need to export the env variable MYSQL_SERVER_PREPARE:
 
-export MYSQL_EMULATED_PREPARE=1
+export MYSQL_SERVER_PREPARE=1
+
+Test first without server side prepare, then with.
 
 
 =item mysql_embedded_options
@@ -801,11 +796,16 @@ $testdsn="DBI:mysqlEmb:database=test;mysql_embedded_groups=embedded_server,commo
     @dbs = $dbh->func('_ListDBs');
 
 Returns a list of all databases managed by the MySQL daemon
-running on C<$hostname>, port C<$port>. This is a legacy
-method.  Instead, you should use the portable method
+running on C<$hostname>, port C<$port>. This method
+is rarely needed for databases running on C<localhost>: You should
+use the portable method
 
     @dbs = DBI->data_sources("mysql");
 
+whenever possible. It is a design problem of this method, that there's
+no way of supplying a host name or port number to C<data_sources>, that's
+the only reason why we still support C<ListDBs>. :-(
+
 =back
 
 
@@ -879,7 +879,7 @@ The DBD::mysql driver supports the following attributes of database
 handles (read only):
 
   $errno = $dbh->{'mysql_errno'};
-  $error = $dbh->{'mysql_error'};
+  $error = $dbh->{'mysql_error};
   $info = $dbh->{'mysql_hostinfo'};
   $info = $dbh->{'mysql_info'};
   $insertid = $dbh->{'mysql_insertid'};
@@ -952,23 +952,6 @@ for $dbh using several ways:
 It is possible to set/unset the C<mysql_use_result> attribute after 
 creation of statement handle. See below.
 
-=item mysql_enable_utf8
-
-This attribute determines whether DBD::mysql should assume strings
-stored in the database are utf8.  This feature defaults to off.
-
-When set, a data retrieved from a textual column type (char, varchar,
-etc) will have the UTF-8 flag turned on if necessary.  This enables
-character semantics on that string.  You will also need to ensure that
-your database / table / column is configured to use UTF8.  See Chapter
-10 of the mysql manual for details.
-
-Additionally, turning on this flag tells MySQL that incoming data should
-be treated as UTF-8.  This will only take effect if used as part of the
-call to connect().  If you turn the flag on after connecting, you will
-need to issue the command C<SET NAMES utf8> to get the same effect.
-
-This option is experimental and may change in future versions.
 
 =head1 STATEMENT HANDLES
 
@@ -1207,67 +1190,350 @@ indication of such loss.
 
 =back
 
+
+=head1 SQL EXTENSIONS
+
+Certain metadata functions of MySQL that are available on the
+C API level, haven't been implemented here. Instead they are implemented
+as "SQL extensions" because they return in fact nothing else but the
+equivalent of a statement handle. These are:
+
 =over
 
-=head1 MULTIPLE RESULT SETS
+=item LISTFIELDS $table
 
-As of version 3.0002_5, DBD::mysql supports multiple result sets (Thanks
-to Guy Harrison!). This is the first release of this functionality, so 
-there may be issues. Please report bugs if you run into them!
+Returns a statement handle that describes the columns of $table.
+Ses the docs of mysql_list_fields (C API) for details.
 
-The basic usage of multiple result sets is
+=back
 
-  do 
-  {
-    while (@row= $sth->fetchrow_array())
-    {
-      do stuff;
-    }
-  } while ($sth->more_results)
 
-An example would be:
 
-  $dbh->do("drop procedure if exists someproc") or print $DBI::errstr;
+=head1 COMPATIBILITY ALERT
 
-  $dbh->do("create procedure somproc() deterministic
-   begin
-   declare a,b,c,d int;
-   set a=1;
-   set b=2;
-   set c=3;
-   set d=4;
-   select a, b, c, d;
-   select d, c, b, a;
-   select b, a, c, d;
-   select c, b, d, a;
-  end") or print $DBI::errstr;
+The statement attribute I<TYPE> has changed its meaning, as of
+DBD::mysql 2.0119. Formerly it used to be the an array
+of native engine's column types, but it is now an array of
+portable SQL column types. The old attribute is still available
+as I<mysql_type>.
 
-  $sth=$dbh->prepare('call someproc()') || 
-  die $DBI::err.": ".$DBI::errstr;
+DBD::mysql is a moving target, due to a number of reasons:
 
-  $sth->execute || die DBI::err.": ".$DBI::errstr; $rowset=0;
-  do {
-    print "\nRowset ".++$i."\n---------------------------------------\n\n";
-    foreach $colno (0..$sth->{NUM_OF_FIELDS}) {
-      print $sth->{NAME}->[$colno]."\t";
-    }
-    print "\n";
-    while (@row= $sth->fetchrow_array())  {
-      foreach $field (0..$#row) {
-        print $row[$field]."\t";
-      }
-      print "\n";
-    }
-  } until (!$sth->more_results)
- 
-For more examples, please see the eg/ directory. This is where helpful
-DBD::mysql code snippits will be added in the future.
+=over
 
-=head2 Issues with Multiple result sets
+=item -
 
-So far, the main issue is if your result sets are "jagged", meaning, the
-number of columns of your results vary. Varying numbers of columns could
-result in your script crashing. This is something that will be fixed soon.
+Of course we have to conform the DBI guidelines and developments.
+
+=item -
+
+We have to keep track with the latest MySQL developments.
+
+=item -
+
+And, surprisingly, we have to be as close to ODBC as possible: This is
+due to the current direction of DBI.
+
+=item -
+
+And, last not least, as any tool it has a little bit life of its own.
+
+=back
+
+This means that a lot of things had to and have to be changed.
+As I am not interested in maintaining a lot of compatibility kludges,
+which only increase the drivers code without being really usefull,
+I did and will remove some features, methods or attributes.
+
+To ensure a smooth upgrade, the following policy will be applied:
+
+=over
+
+=item Obsolete features
+
+The first step is to declare something obsolete. This means, that no code
+is changed, but the feature appears in the list of obsolete features. See
+L<Obsolete Features> below.
+
+=item Deprecated features
+
+If the feature has been obsolete for quite some time, typically in the
+next major stable release, warnings will be inserted in the code. You
+can suppress these warnings by setting
+
+    $DBD::mysql = 1;
+
+In the docs the feature will be moved from the list of obsolete features
+to the list of deprecated features. See L<Deprecated Features> below.
+
+=item Removing features
+
+Finally features will be removed silently in the next major stable
+release. The feature will be shown in the list of historic features.
+See L<Historic Features> below.
+
+=back
+
+Example: The statement handle attribute
+
+    $sth->{'LENGTH'}
+
+was declared obsolete in DBD::mysql 2.00xy. It was considered
+deprecated in DBD::mysql 2.02xy and removed in 2.04xy.
+
+
+=head2 Obsolete Features
+
+=over
+
+=item Database handle attributes
+
+The following database handle attributes are declared obsolete
+in DBD::mysql 2.09. They will be deprecated in 2.11 and removed
+in 2.13.
+
+=over
+
+=item C<$dbh->{'errno'}>
+
+Replaced by C<$dbh->{'mysql_errno'}>
+
+=item C<$dbh->{'errmsg'}>
+
+Replaced by C<$dbh->{'mysql_error'}>
+
+=item C<$dbh->{'hostinfo'}>
+
+Replaced by C<$dbh->{'mysql_hostinfo'}>
+
+=item C<$dbh->{'info'}>
+
+Replaced by C<$dbh->{'mysql_info'}>
+
+=item C<$dbh->{'protoinfo'}>
+
+Replaced by C<$dbh->{'mysql_protoinfo'}>
+
+=item C<$dbh->{'serverinfo'}>
+
+Replaced by C<$dbh->{'mysql_serverinfo'}>
+
+=item C<$dbh->{'stats'}>
+
+Replaced by C<$dbh->{'mysql_stat'}>
+
+=item C<$dbh->{'thread_id'}>
+
+Replaced by C<$dbh->{'mysql_thread_id'}>
+
+=back
+
+=back
+
+
+=head2 Deprecated Features
+
+=over
+
+=item _ListTables
+
+Replace with the standard DBI method C<$dbh->tables()>. See also
+C<$dbh->table_info()>. Portable applications will prefer
+
+    @tables = map { $_ =~ s/.*\.//; $_ } $dbh->tables()
+
+because, depending on the engine, the string "user.table" will be
+returned, user being the table owner. The method will be removed
+in DBD::mysql version 2.11xy.
+
+=back
+
+
+=head2 Historic Features
+
+=over
+
+=item _CreateDB
+
+=item _DropDB
+
+The methods
+
+    $dbh->func($db, '_CreateDB');
+    $dbh->func($db, '_DropDB');
+
+have been used for creating or dropping databases. They have been removed
+in 1.21_07 in favour of
+
+    $drh->func("createdb", $dbname, $host, "admin")
+    $drh->func("dropdb", $dbname, $host, "admin")
+
+=item _ListFields
+
+The method
+
+    $sth = $dbh->func($table, '_ListFields');
+
+has been used to list a tables columns names, types and other attributes.
+This method has been removed in 1.21_07 in favour of
+
+    $sth = $dbh->prepare("LISTFIELDS $table");
+
+=item _ListSelectedFields
+
+The method
+
+    $sth->func('_ListSelectedFields');
+
+use to return a hash ref of attributes like 'IS_NUM', 'IS_KEY' and so
+on. These attributes are now accessible via
+
+    $sth->{'mysql_is_num'};
+    $sth->{'mysql_is_key'};
+
+and so on. Thus the method has been removed in 1.21_07.
+
+=item _NumRows
+
+The method
+
+    $sth->func('_NumRows');
+
+used to be equivalent to
+
+    $sth->rows();
+
+and has been removed in 1.21_07.
+
+=item _InsertID
+
+The method
+
+    $dbh->func('_InsertID');
+
+used to be equivalent with
+
+    $dbh->{'mysql_insertid'};
+
+=item Statement handle attributes
+
+=over
+
+=item affected_rows
+
+Replaced with $sth->{'mysql_affected_rows'} or the result
+of $sth->execute().
+
+=item format_default_size
+
+Replaced with $sth->{'PRECISION'}.
+
+=item format_max_size
+
+Replaced with $sth->{'mysql_max_length'}.
+
+=item format_type_name
+
+Replaced with $sth->{'TYPE'} (portable) or
+$sth->{'mysql_type_name'} (MySQL specific).
+
+=item format_right_justify
+
+Replaced with $sth->->{'TYPE'} (portable) or
+$sth->{'mysql_is_num'} (MySQL specific).
+
+=item insertid
+
+Replaced with $sth->{'mysql_insertid'}.
+
+=item IS_BLOB
+
+Replaced with $sth->{'TYPE'} (portable) or
+$sth->{'mysql_is_blob'} (MySQL specific).
+
+=item is_blob
+
+Replaced with $sth->{'TYPE'} (portable) or
+$sth->{'mysql_is_blob'} (MySQL specific).
+
+=item IS_PRI_KEY
+
+Replaced with $sth->{'mysql_is_pri_key'}.
+
+=item is_pri_key
+
+Replaced with $sth->{'mysql_is_pri_key'}.
+
+=item IS_NOT_NULL
+
+Replaced with $sth->{'NULLABLE'} (do not forget to invert
+the boolean values).
+
+=item is_not_null
+
+Replaced with $sth->{'NULLABLE'} (do not forget to invert
+the boolean values).
+
+=item IS_NUM
+
+Replaced with $sth->{'TYPE'} (portable) or
+$sth->{'mysql_is_num'} (MySQL specific).
+
+=item is_num
+
+Replaced with $sth->{'TYPE'} (portable) or
+$sth->{'mysql_is_num'} (MySQL specific).
+
+=item IS_KEY
+
+Replaced with $sth->{'mysql_is_key'}.
+
+=item is_key
+
+Replaced with $sth->{'mysql_is_key'}.
+
+=item MAXLENGTH
+
+Replaced with $sth->{'mysql_max_length'}.
+
+=item maxlength
+
+Replaced with $sth->{'mysql_max_length'}.
+
+=item LENGTH
+
+Replaced with $sth->{'PRECISION'} (portable) or
+$sth->{'mysql_length'} (MySQL specific)
+
+=item length
+
+Replaced with $sth->{'PRECISION'} (portable) or
+$sth->{'mysql_length'} (MySQL specific)
+
+=item NUMFIELDS
+
+Replaced with $sth->{'NUM_OF_FIELDS'}.
+
+=item numfields
+
+Replaced with $sth->{'NUM_OF_FIELDS'}.
+
+=item NUMROWS
+
+Replaced with the result of $sth->execute() or
+$sth->{'mysql_affected_rows'}.
+
+=item TABLE
+
+Replaced with $sth->{'mysql_table'}.
+
+=item table
+
+Replaced with $sth->{'mysql_table'}.
+
+=back
+
+=back
 
 
 =head1 MULTITHREADING
@@ -1317,35 +1583,36 @@ you a lot of questions. If you finally receive the CPAN prompt, enter
 If this fails (which may be the case for a number of reasons, for
 example because you are behind a firewall or don't have network
 access), you need to do a manual installation. First of all you
-need to fetch the modules from CPAN search
+need to fetch the archives from any CPAN mirror, for example
 
-   http://search.cpan.org/ 
+  ftp://ftp.funet.fi/pub/languages/perl/CPAN/modules/by-module
 
-The following modules are required
+The following archives are required (version numbers may have
+changed, I choose those which are current as of this writing):
 
-  DBI
-  Data::ShowTable
-  DBD::mysql
+  DBI/DBI-1.15.tar.gz
+  Data/Data-ShowTable-3.3.tar.gz
+  DBD/DBD-mysql-2.1001.tar.gz
 
-Then enter the following commands (note - versions are just examples):
+Then enter the following commands:
 
-  gzip -cd DBI-(version).tar.gz | tar xf -
-  cd DBI-(version)
+  gzip -cd DBI-1.15.tar.gz | tar xf -
+  cd DBI-1.15
   perl Makefile.PL
   make
   make test
   make install
 
   cd ..
-  gzip -cd Data-ShowTable-(version).tar.gz | tar xf -
+  gzip -cd Data-ShowTable-3.3.tar.gz | tar xf -
   cd Data-ShowTable-3.3
   perl Makefile.PL
   make
-  make install
+  make install  # Don't try make test, the test suite is broken
 
   cd ..
-  gzip -cd DBD-mysql-(version)-tar.gz | tar xf -
-  cd DBD-mysql-(version)
+  gzip -cd DBD-mysql-2.1001.tar.gz | tar xf -
+  cd DBD-mysql-2.1001
   perl Makefile.PL
   make
   make test
@@ -1382,9 +1649,10 @@ Otherwise you definitely *need* a C compiler. And it *must* be the same
 compiler that was being used for compiling Perl itself. If you don't
 have a C compiler, the file README.win32 from the Perl source
 distribution tells you where to obtain freely distributable C compilers
-like egcs or gcc. The Perl sources are available via CPAN search
+like egcs or gcc. The Perl sources are available on any CPAN mirror in
+the src directory, for example
 
-  http://search.cpan.org
+    ftp://ftp.funet.fi/pub/languages/perl/CPAN/src/latest.tar.gz
 
 I recommend using the win32clients package for installing DBD::mysql
 under Win32, available for download on www.tcx.se. The following steps
@@ -1498,9 +1766,10 @@ in the PPM program.
 
 The current version of B<DBD::mysql> is almost completely written
 by Jochen Wiedmann, and is now being maintained by
-Patrick Galbraith (I<patg@mysql.com>). 
-The first version's author was Alligator Descartes, who was aided
-and abetted by Gary Shea, Andreas Knig and Tim Bunce amongst others.
+Rudy Lippan (I<rlippan@remotelinux.com>). The first version's author
+was Alligator Descartes (I<descarte@symbolstone.org>), who has been
+aided and abetted by Gary Shea, Andreas Knig and Tim Bunce
+amongst others.
 
 The B<Mysql> module was originally written by Andreas Knig
 <koenig@kulturbox.de>. The current version, mainly an emulation
@@ -1510,9 +1779,7 @@ layer, is from Jochen Wiedmann.
 =head1 COPYRIGHT
 
 
-This module is 
-Large Portions Copyright (c) 2004-2006 MySQL Patrick Galbraith, Alexey Stroganov,
-Large Portions Copyright (c) 2003-2005 Rudolf Lippan; Large Portions 
+This module is Copyright (c) 2003 Rudolf Lippan; Large Portions 
 Copyright (c) 1997-2003 Jochen Wiedmann, with code portions 
 Copyright (c)1994-1997 their original authors This module is
 released under the same license as Perl itself. See the Perl README
@@ -1525,33 +1792,37 @@ This module is maintained and supported on a mailing list,
 
     perl@lists.mysql.com
 
-To subscribe to this list, go to
+To subscribe to this list, send a mail to
 
-http://lists.mysql.com/perl?sub=1
+    perl-subscribe@lists.mysql.com
+
+or
+
+    perl-digest-subscribe@lists.mysql.com
 
 Mailing list archives are available at
 
-http://lists.mysql.com/perl
+    http://www.progressive-comp.com/Lists/?l=msql-mysql-modules
+
 
 Additionally you might try the dbi-user mailing list for questions about
 DBI and its modules in general. Subscribe via
 
-dbi-users-subscribe@perl.org
+    http://www.fugue.com/dbi
 
 Mailing list archives are at
 
-http://groups.google.com/group/perl.dbi.users?hl=en&lr=
+     http://www.rosat.mpe-garching.mpg.de/mailing-lists/PerlDB-Interest/
+     http://outside.organic.com/mail-archives/dbi-users/
+     http://www.coe.missouri.edu/~faq/lists/dbi.html
 
-Also, the main DBI site is at
-
-http://dbi.perl.org/
 
 =head1 ADDITIONAL DBI INFORMATION
 
 Additional information on the DBI project can be found on the World
 Wide Web at the following URL:
 
-    http://dbi.perl.org
+    http://www.symbolstone.org/technology/perl/DBI
 
 where documentation, pointers to the mailing lists and mailing list
 archives and pointers to the most current versions of the modules can
@@ -1563,16 +1834,6 @@ Information on the DBI interface itself can be gained by typing:
 
 right now!
 
-
-=head1 BUG REPORTING, ENHANCEMENT/FEATURE REQUESTS
-
-Please report bugs, including all the information needed
-such as DBD::mysql version, MySQL version, OS type/version, etc
-to this link:
-
-http://bugs.mysql.com/
-
-
 =cut
 
 
